@@ -173,8 +173,14 @@ static int unix_pass(char *name, char *passwd)
 	/*
 	 *      Check if password has expired.
 	 */
+
+#if defined(bsdi)
+        if (spwd && spwd->pw_expire > 0 &&
+            (time(NULL) / 86400) > spwd->pw_expire) {
+#else
 	if (spwd && spwd->sp_expire > 0 &&
 	    (time(NULL) / 86400) > spwd->sp_expire) {
+#endif
 		log(L_AUTH, "unix_pass: [%s]: password has expired", name);
 		return -1;
 	}
@@ -521,7 +527,7 @@ int rad_auth_init(AUTH_REQ *authreq, int activefd) {
 	/*
 	 *	Add any specific attributes for this username.
 	 */
-	hints_setup(authreq->request);
+	hints_setup(authreq);
 
 /* FIXME: find out what this logs */
 /*	if (log_auth_detail)
@@ -551,8 +557,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	VALUE_PAIR	*check_item;
 	VALUE_PAIR	*reply_item;
 	VALUE_PAIR	*auth_item;
-	VALUE_PAIR	*user_check;
-	VALUE_PAIR	*user_reply;
 	int		result, r;
 	char		pw_digest[16];
 	char		userpass[AUTH_STRING_LEN];
@@ -566,8 +570,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	char		dbpass[128];
 	int 		i;
 
-	user_check = NULL;
-	user_reply = NULL;
 	user_msg = NULL;
 	userpass[0] = 0;
 	dbpass[0] = 0;
@@ -592,29 +594,27 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 */
 	if (authreq->server_code == PW_AUTHENTICATION_REJECT ||
 	    authreq->server_code == PW_AUTHENTICATION_ACK) {
-		if ((user_check = malloc(sizeof(VALUE_PAIR))) == NULL) {
+		if ((authreq->check_pairs = malloc(sizeof(VALUE_PAIR))) == NULL) {
 			log(L_ERR|L_CONS, "no memory");
 			exit(1);
 		}
-		memset(user_check, 0, sizeof(VALUE_PAIR));
-		user_check->attribute = PW_AUTHTYPE;
-		strcpy(user_check->name, "Auth-Type");
-		user_check->type = PW_TYPE_INTEGER;
-		user_check->length = 4;
+		memset(authreq->check_pairs, 0, sizeof(VALUE_PAIR));
+		authreq->check_pairs->attribute = PW_AUTHTYPE;
+		strcpy(authreq->check_pairs->name, "Auth-Type");
+		authreq->check_pairs->type = PW_TYPE_INTEGER;
+		authreq->check_pairs->length = 4;
 	}
 	if (authreq->server_code == PW_AUTHENTICATION_REJECT)
-		user_check->lvalue = PW_AUTHTYPE_REJECT;
+		authreq->check_pairs->lvalue = PW_AUTHTYPE_REJECT;
 	if (authreq->server_code == PW_AUTHENTICATION_ACK)
-		user_check->lvalue = PW_AUTHTYPE_ACCEPT;
+		authreq->check_pairs->lvalue = PW_AUTHTYPE_ACCEPT;
 
 	if (authreq->server_reply)
-		user_reply = authreq->server_reply;
+		authreq->reply_pairs = authreq->server_reply;
 
 
 	if (authreq->server_code) {
 
-		VALUE_PAIR	*realm_check = NULL;
-
 		auth_item = pairfind(authreq->request, PW_PASSWORD);
 		/*
 		 *	Decrypt the password.
@@ -626,19 +626,19 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			string[AUTH_PASS_LEN] = '\0';
 			strcpy(userpass, string);
 		}
-		sql_getvpdata(socket, sql->config->sql_groupcheck_table, &realm_check, authreq->realm, PW_VP_REALMDATA);
-		sql_getvpdata(socket, sql->config->sql_groupreply_table, &user_reply, authreq->realm, PW_VP_REALMDATA);
+		sql_getvpdata(socket, sql->config->sql_groupcheck_table, &authreq->check_pairs, authreq->realm, PW_VP_REALMDATA);
+		sql_getvpdata(socket, sql->config->sql_groupreply_table, &authreq->reply_pairs, authreq->realm, PW_VP_REALMDATA);
 
 		if (authreq->server_code == PW_AUTHENTICATION_ACK) {
 			result = 0;
 
-			if((result = check_expiration(realm_check, umsg, &user_msg)) < 0) {
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd); 
+			if((result = check_expiration(authreq->check_pairs, umsg, &user_msg)) < 0) {
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd); 
 					log(L_AUTH, "Realm Expired: [%s] (%s)", authreq->realm, auth_name(authreq, 1));
 					return -1;
 			}
 
-			if (result >= 0 && (check_item = pairfind(realm_check, PW_SIMULTANEOUS_USE)) != NULL) {
+			if (result >= 0 && (check_item = pairfind(authreq->check_pairs, PW_SIMULTANEOUS_USE)) != NULL) {
 
 				SQL_ROW		row;
 				char		querystr[256];
@@ -653,12 +653,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d simultaneous logins\r\n\n", (int)check_item->lvalue);
 					user_msg = umsg;
 					result = -1;
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Realm multiple logins: [%s] (%s) max. %d", authreq->realm, auth_name(authreq, 1), check_item->lvalue);
 				}
 			}
 
-			if (result >=0 && (check_item = pairfind(realm_check, PW_TOTAL_TIME_LIMIT)) != NULL) {
+			if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_TOTAL_TIME_LIMIT)) != NULL) {
 
 				SQL_ROW		row;
 				char		querystr[256];
@@ -680,11 +680,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					sprintf(umsg, "\r\nYour realm has reached it's total limit of %d seconds\r\n\n", (int)check_item->lvalue);
 					user_msg = umsg;
 					result = -1;
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Realm usage total limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
 				}
 			
-				if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+				if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 					diff = allowed - used;
 					if (diff < reply_item->lvalue)
 						reply_item->lvalue = diff;
@@ -699,12 +699,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					reply_item->type = PW_TYPE_INTEGER;
 					reply_item->length = 4;
 					reply_item->lvalue = allowed - used;
-					pairadd(&user_reply, reply_item);
+					pairadd(&authreq->reply_pairs, reply_item);
 				}
 
 			}
 
-			if (result >=0 && (check_item = pairfind(realm_check, PW_MONTHLY_TIME_LIMIT)) != NULL) {
+			if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_MONTHLY_TIME_LIMIT)) != NULL) {
 
 				SQL_ROW		row;
 				char		querystr[256];
@@ -726,11 +726,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d seconds\r\n\n", (int)check_item->lvalue);
 					user_msg = umsg;
 					result = -1;
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Realm usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
 				}
 			
-				if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+				if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 					diff = allowed - used;
 					if (diff < reply_item->lvalue)
 						reply_item->lvalue = diff;
@@ -745,12 +745,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					reply_item->type = PW_TYPE_INTEGER;
 					reply_item->length = 4;
 					reply_item->lvalue = allowed - used;
-					pairadd(&user_reply, reply_item);
+					pairadd(&authreq->reply_pairs, reply_item);
 				}
 
 			}
 			/* Old attribute in hours format */
-			if (result >=0 && (check_item = pairfind(realm_check, PW_MAX_HOURS)) != NULL) {
+			if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_MAX_HOURS)) != NULL) {
 
 				SQL_ROW		row;
 				char		querystr[256];
@@ -772,11 +772,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d hours\r\n\n", (int)check_item->lvalue);
 					user_msg = umsg;
 					result = -1;
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Realm usage limit of %d hours reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
 				}
 			
-				if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+				if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 					diff = allowed - used;
 					if (diff < reply_item->lvalue)
 						reply_item->lvalue = diff;
@@ -791,13 +791,13 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					reply_item->type = PW_TYPE_INTEGER;
 					reply_item->length = 4;
 					reply_item->lvalue = allowed - used;
-					pairadd(&user_reply, reply_item);
+					pairadd(&authreq->reply_pairs, reply_item);
 				}
 
 			}
 
 			if (result >= 0 &&
-			   (check_item = pairfind(realm_check, PW_LOGIN_TIME)) != NULL) {
+			   (check_item = pairfind(authreq->check_pairs, PW_LOGIN_TIME)) != NULL) {
 	
 				/*
 				 *	Authentication is OK. Now see if this
@@ -812,7 +812,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					user_msg =
 					"You are calling outside your allowed timespan\r\n";
 					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-						user_reply, user_msg, activefd);
+						authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Realm Outside allowed timespan: [%s]"
 						   " (from nas %s) time allowed: %s",
 							authreq->realm,
@@ -822,7 +822,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					/*
 					 *	User is allowed, but set Session-Timeout.
 					 */
-					if ((reply_item = pairfind(user_reply,
+					if ((reply_item = pairfind(authreq->reply_pairs,
 					    PW_SESSION_TIMEOUT)) != NULL) {
 						if (reply_item->lvalue > r)
 							reply_item->lvalue = r;
@@ -837,19 +837,17 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 						reply_item->type = PW_TYPE_INTEGER;
 						reply_item->length = 4;
 						reply_item->lvalue = r;
-						pairadd(&user_reply, reply_item);
+						pairadd(&authreq->reply_pairs, reply_item);
 					}
 				}
 			}
 			if (result < 0) {
 				authfree(authreq);
-				pairfree(realm_check);
-				pairfree(user_reply);
 				return 0;
 			}
 
 		} else if (authreq->server_code == PW_AUTHENTICATION_REJECT) {
-			pairfree(realm_check);
+			pairfree(authreq->check_pairs);
 			result = -1;
 		}
 
@@ -858,24 +856,17 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		/*
 		 *	Get the user from the database
 		 */
-		if (user_find(namepair->strvalue, authreq->request,
-		   &user_check, &user_reply, socket) != 0) {
-			log(L_AUTH, "Invalid user: [%s] (%s)",
-				namepair->strvalue, auth_name(authreq, 1));
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-				NULL, NULL, activefd);
+		if (user_find(namepair->strvalue, authreq, socket) != 0) {
+			log(L_AUTH, "Invalid user: [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, NULL, NULL, activefd);
 			authfree(authreq);
-			pairfree(user_check);
-			pairfree(user_reply);
 			return -1;
 		}
 
-		if (paircmp(authreq->request, user_check) != 0) {
+		if (paircmp(authreq->request, authreq->check_pairs) != 0) {
 			log(L_AUTH, "Check list does not match request list [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, NULL, NULL, activefd);
 			authfree(authreq);
-			pairfree(user_check);
-			pairfree(user_reply);
 			return -1;
 		}
 			
@@ -884,15 +875,13 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		 *	Validate the user
 		 */
 		do {
-			if ((result = check_expiration(user_check, umsg, &user_msg))<0) {
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd); 
+			if ((result = check_expiration(authreq->check_pairs, umsg, &user_msg))<0) {
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd); 
 					log(L_AUTH, "Account Expired: [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
 					authfree(authreq);
-					pairfree(user_check);
-					pairfree(user_reply);
 					return -1;
 			}
-			if (result >= 0 && (check_item = pairfind(user_check, PW_ACTIVATION)) != NULL) {
+			if (result >= 0 && (check_item = pairfind(authreq->check_pairs, PW_ACTIVATION)) != NULL) {
 
 				time_t		now;
 				struct tm	*actdate;
@@ -904,23 +893,21 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					sprintf(umsg, "\r\nYour account is not yet activated\r\n\n");
 					user_msg = umsg;
 					result = -1;
-					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 					log(L_ERR, "Account not activated until %d-%d-%d: [%s] (%s)", actdate->tm_year+1900, 
 						actdate->tm_mon+1, actdate->tm_mday, namepair->strvalue, auth_name(authreq, 1));
 				}
 			}
 
 
-			result = rad_check_password(authreq, activefd, user_check, namepair, pw_digest, &user_msg, userpass , dbpass);
+			result = rad_check_password(authreq, activefd, authreq->check_pairs, namepair, pw_digest, &user_msg, userpass , dbpass);
 
 			if (result > 0) {
 				authfree(authreq);
-				pairfree(user_check);
-				pairfree(user_reply);
 				return -1;
 			}
 			if (result == -2) {
-				if ((reply_item = pairfind(user_reply, PW_REPLY_MESSAGE)) != NULL)
+				if ((reply_item = pairfind(authreq->reply_pairs, PW_REPLY_MESSAGE)) != NULL)
 					user_msg = reply_item->strvalue;
 			}
 		} while(0);
@@ -931,13 +918,13 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		/*
 		 *	Failed to validate the user.
 		 */
-		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 		if (log_auth)
 			log(L_AUTH, "Login incorrect: [%s/%s]%s (%s)", namepair->strvalue, userpass, dbpass, auth_name(authreq, 1));
 	}
 
 
-	if (result >=0 && (check_item = pairfind(user_check, PW_TOTAL_TIME_LIMIT)) != NULL) {
+	if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_TOTAL_TIME_LIMIT)) != NULL) {
 
 		SQL_ROW		row;
 		char		querystr[256];
@@ -959,11 +946,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			sprintf(umsg, "\r\nYour account has reached it total limit of %d seconds\r\n\n", (int)check_item->lvalue);
 			user_msg = umsg;
 			result = -1;
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 			log(L_ERR, "Total usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, namepair->strvalue, auth_name(authreq, 1));
 		}
 			
-		if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+		if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 			diff = allowed - used;
 			if (diff < reply_item->lvalue)
 				reply_item->lvalue = diff;
@@ -978,12 +965,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			reply_item->type = PW_TYPE_INTEGER;
 			reply_item->length = 4;
 			reply_item->lvalue = allowed - used;
-			pairadd(&user_reply, reply_item);
+			pairadd(&authreq->reply_pairs, reply_item);
 		}
 
 	}
 
-	if (result >=0 && (check_item = pairfind(user_check, PW_MONTHLY_TIME_LIMIT)) != NULL) {
+	if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_MONTHLY_TIME_LIMIT)) != NULL) {
 
 		SQL_ROW		row;
 		char		querystr[256];
@@ -1005,11 +992,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			sprintf(umsg, "\r\nYour account has reached it limit of %d seconds\r\n\n", (int)check_item->lvalue);
 			user_msg = umsg;
 			result = -1;
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 			log(L_ERR, "Usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, namepair->strvalue, auth_name(authreq, 1));
 		}
 			
-		if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+		if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 			diff = allowed - used;
 			if (diff < reply_item->lvalue)
 				reply_item->lvalue = diff;
@@ -1024,12 +1011,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			reply_item->type = PW_TYPE_INTEGER;
 			reply_item->length = 4;
 			reply_item->lvalue = allowed - used;
-			pairadd(&user_reply, reply_item);
+			pairadd(&authreq->reply_pairs, reply_item);
 		}
 
 	}
 	/* Old attribute in hours format */
-	if (result >=0 && (check_item = pairfind(user_check, PW_MAX_HOURS)) != NULL) {
+	if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_MAX_HOURS)) != NULL) {
 
 		SQL_ROW		row;
 		char		querystr[256];
@@ -1051,11 +1038,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			sprintf(umsg, "\r\nYour account has reached it limit of %d hours\r\n\n", (int)check_item->lvalue);
 			user_msg = umsg;
 			result = -1;
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 			log(L_ERR, "Usage limit of %d hours reached: [%s] (%s)", (int)check_item->lvalue, namepair->strvalue, auth_name(authreq, 1));
 		}
 			
-		if (result >=0 && (reply_item = pairfind(user_reply, PW_SESSION_TIMEOUT)) != NULL) {
+		if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
 			diff = allowed - used;
 			if (diff < reply_item->lvalue)
 				reply_item->lvalue = diff;
@@ -1070,14 +1057,14 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			reply_item->type = PW_TYPE_INTEGER;
 			reply_item->length = 4;
 			reply_item->lvalue = allowed - used;
-			pairadd(&user_reply, reply_item);
+			pairadd(&authreq->reply_pairs, reply_item);
 		}
 
 	}
 
 
 	if (result >= 0 &&
-	   (check_item = pairfind(user_check, PW_SIMULTANEOUS_USE)) != NULL) {
+	   (check_item = pairfind(authreq->check_pairs, PW_SIMULTANEOUS_USE)) != NULL) {
 #if 0 /* DEBUG */
 		VALUE_PAIR *tmp;
 		char ipno[32];
@@ -1094,7 +1081,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			} else {
 				user_msg = "\r\nYou are already logged in - access denied\r\n\n";
 			}
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 		log(L_ERR, "Multiple logins: [%s] (%s) max. %d%s", namepair->strvalue, auth_name(authreq, 1),
 				check_item->lvalue, r == 2 ? " [MPP attempt]" : "");
 			result = -1;
@@ -1108,7 +1095,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	}
 
 	if (result >= 0 &&
-	   (check_item = pairfind(user_check, PW_LOGIN_TIME)) != NULL) {
+	   (check_item = pairfind(authreq->check_pairs, PW_LOGIN_TIME)) != NULL) {
 
 		/*
 		 *	Authentication is OK. Now see if this
@@ -1122,14 +1109,14 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			result = -1;
 			user_msg =
 			"You are calling outside your allowed timespan\r\n";
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
 			log(L_ERR, "Outside allowed timespan: [%s]" " (%s) time allowed: %s", namepair->strvalue,
 					auth_name(authreq, 1), check_item->strvalue);
 		} else if (r > 0) {
 			/*
 			 *	User is allowed, but set Session-Timeout.
 			 */
-			if ((reply_item = pairfind(user_reply,
+			if ((reply_item = pairfind(authreq->reply_pairs,
 			    PW_SESSION_TIMEOUT)) != NULL) {
 				if (reply_item->lvalue > r)
 					reply_item->lvalue = r;
@@ -1144,7 +1131,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 				reply_item->type = PW_TYPE_INTEGER;
 				reply_item->length = 4;
 				reply_item->lvalue = r;
-				pairadd(&user_reply, reply_item);
+				pairadd(&authreq->reply_pairs, reply_item);
 			}
 		}
 	}
@@ -1154,8 +1141,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 */
 	if (result < 0) {
 		authfree(authreq);
-		pairfree(user_check);
-		pairfree(user_reply);
 		return 0;
 	}
 
@@ -1169,15 +1154,15 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 */
 	exec_program = NULL;
 	exec_wait = 0;
-	if ((auth_item = pairfind(user_reply, PW_EXEC_PROGRAM)) != NULL) {
+	if ((auth_item = pairfind(authreq->reply_pairs, PW_EXEC_PROGRAM)) != NULL) {
 		exec_wait = 0;
 		exec_program = strdup(auth_item->strvalue);
-		pairdelete(&user_reply, PW_EXEC_PROGRAM);
+		pairdelete(&authreq->reply_pairs, PW_EXEC_PROGRAM);
 	}
-	if ((auth_item = pairfind(user_reply, PW_EXEC_PROGRAM_WAIT)) != NULL) {
+	if ((auth_item = pairfind(authreq->reply_pairs, PW_EXEC_PROGRAM_WAIT)) != NULL) {
 		exec_wait = 1;
 		exec_program = strdup(auth_item->strvalue);
-		pairdelete(&user_reply, PW_EXEC_PROGRAM_WAIT);
+		pairdelete(&authreq->reply_pairs, PW_EXEC_PROGRAM_WAIT);
 	}
 
 	/*
@@ -1185,10 +1170,10 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 *	This is nice for certain Exec-Program programs.
 	 */
 	seen_callback_id = 0;
-	if ((auth_item = pairfind(user_reply, PW_CALLBACK_ID)) != NULL) {
+	if ((auth_item = pairfind(authreq->reply_pairs, PW_CALLBACK_ID)) != NULL) {
 		seen_callback_id = 1;
 		ptr = radius_xlate(auth_item->strvalue,
-			authreq->request, user_reply);
+			authreq->request, authreq->reply_pairs);
 		strncpy(auth_item->strvalue, ptr, sizeof(auth_item->strvalue));
 		auth_item->strvalue[sizeof(auth_item->strvalue) - 1] = 0;
 		auth_item->length = strlen(auth_item->strvalue);
@@ -1201,7 +1186,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 */
 	if (exec_program && exec_wait) {
 		if (radius_exec_program(exec_program,
-		    authreq->request, &user_reply, exec_wait, &user_msg) != 0) {
+		    authreq->request, &authreq->reply_pairs, exec_wait, &user_msg) != 0) {
 			/*
 			 *	Error. radius_exec_program() returns -1 on
 			 *	fork/exec errors, or >0 if the exec'ed program
@@ -1210,7 +1195,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			if (user_msg == NULL)
 		user_msg = "\r\nAccess denied (external check failed).";
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-				user_reply, user_msg, activefd);
+				authreq->reply_pairs, user_msg, activefd);
 			if (log_auth) {
 				log(L_AUTH,
 					"Login incorrect: [%s] (%s) "
@@ -1219,8 +1204,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					auth_name(authreq, 1));
 			}
 			authfree(authreq);
-			pairfree(user_check);
-			pairfree(user_reply);
 			return 0;
 		}
 	}
@@ -1236,25 +1219,25 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	 *	(if you knew how I use the exec_wait, you'd understand).
 	 */
 	if (seen_callback_id) {
-		pairdelete(&user_reply, PW_FRAMED_PROTOCOL);
-		pairdelete(&user_reply, PW_FRAMED_IP_ADDRESS);
-		pairdelete(&user_reply, PW_FRAMED_IP_NETMASK);
-		pairdelete(&user_reply, PW_FRAMED_ROUTE);
-		pairdelete(&user_reply, PW_FRAMED_MTU);
-		pairdelete(&user_reply, PW_FRAMED_COMPRESSION);
-		pairdelete(&user_reply, PW_FILTER_ID);
-		pairdelete(&user_reply, PW_PORT_LIMIT);
-		pairdelete(&user_reply, PW_CALLBACK_NUMBER);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_PROTOCOL);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_IP_ADDRESS);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_IP_NETMASK);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_ROUTE);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_MTU);
+		pairdelete(&authreq->reply_pairs, PW_FRAMED_COMPRESSION);
+		pairdelete(&authreq->reply_pairs, PW_FILTER_ID);
+		pairdelete(&authreq->reply_pairs, PW_PORT_LIMIT);
+		pairdelete(&authreq->reply_pairs, PW_CALLBACK_NUMBER);
 	}
 
 	/*
 	 *	Filter Port-Message value through radius_xlate
 	 */
 	if (user_msg == NULL) {
-		if ((reply_item = pairfind(user_reply,
+		if ((reply_item = pairfind(authreq->reply_pairs,
 		    PW_REPLY_MESSAGE)) != NULL) {
 			user_msg = radius_xlate(reply_item->strvalue,
-				authreq->request, user_reply);
+				authreq->request, authreq->reply_pairs);
 			strNcpy(reply_item->strvalue, user_msg,
 				sizeof(reply_item->strvalue));
 			reply_item->length = strlen(reply_item->strvalue);
@@ -1262,7 +1245,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		}
 	}
 
-	rad_send_reply(PW_AUTHENTICATION_ACK, authreq, user_reply, user_msg, activefd);
+	rad_send_reply(PW_AUTHENTICATION_ACK, authreq, authreq->reply_pairs, user_msg, activefd);
 	if (log_auth) {
 
 		UINT4	curtime = (UINT4)time(NULL);
@@ -1274,13 +1257,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		/*
 		 *	No need to check the exit status here.
 		 */
-		radius_exec_program(exec_program, authreq->request, &user_reply, exec_wait, NULL);
+		radius_exec_program(exec_program, authreq->request, &authreq->reply_pairs, exec_wait, NULL);
 	}
 
 	if (exec_program) free(exec_program);
 	authfree(authreq);
-	pairfree(user_check);
-	pairfree(user_reply);
 
 	return 0;
 }
