@@ -10,16 +10,18 @@
 #
 # Returns:	0 = no duplicate, 1 = duplicate, >1 = error.
 #
-# Version:	@(#)checkrad.pl  1.18  25-Jul-1999  miquels@cistron.nl
+# Version:	@(#)checkrad.pl  1.21  04-Jan-2000  miquels@cistron.nl
 #
 #		livingston_snmp  1.1	Author: miquels@cistron.nl
 #		portslave_finger 1.0	Author: miquels@cistron.nl
 #		max40xx_finger	 1.0	Author: costa@mdi.ca
+#		ascend_snmp      1.1    Author: blaz@amis.net
 #		computone_finger 1.1	Author: costa@mdi.ca
 #		sub tc_tccheck	 1.1	Author: alexisv@compass.com.ph
-#		pathras_telnet	 1.1	Author: accdias@sst.com.br
+#		cyclades_telnet	 1.1	Author: accdias@sst.com.br
 #		patton_snmp	 1.0	Author: accdias@sst.com.br
-#		digitro_rusers	 1.0	Author: accdias@sst.com.br
+#		digitro_rusers	 1.1	Author: accdias@sst.com.br
+#		cyclades_snmp	 1.0	Author: accdias@sst.com.br
 #		usrhiper_snmp    1.0    Author: igor@ipass.net
 #		multitech_snmp   1.0    Author: ehonzay@willmar.com
 #		netserver_telnet 1.0	Author: mts@interplanet.es
@@ -27,6 +29,8 @@
 #
 #	Config: $debug is the file you want to put debug messages in
 #		$snmpget is the location of your ``snmpget'' program
+#		$snmpwalk is the location of your ``snmpwalk'' program
+#		$rusers is the location of your ``rusers'' program
 #		$naspass is the location of your NAS admin password file
 #
 
@@ -40,8 +44,9 @@ $debug		= "";
 #$debug		= "$logdir/checkrad.log";
 
 $snmpget	= "/usr/bin/snmpget";
-$naspass	= "$raddbdir/naspasswd";
+$snmpwalk	= "/usr/bin/snmpwalk";
 $rusers		= "/usr/bin/rusers";
+$naspass	= "$raddbdir/naspasswd";
 
 #
 #	PM3:	$lv_offs is where the last S port is before one or two
@@ -332,6 +337,37 @@ sub max40xx_finger {
 	close FD;
 }
 
+#
+#	Check an Ascend Max4000 or similar model via SNMP
+#
+#	Author: Blaz Zupan of Medinet <blaz@amis.net>
+#
+$asm   = '.iso.org.dod.internet.private.enterprises.529';
+sub ascend_snmp {
+	my $sess_id;
+	my $l1, $l2;
+
+	$l1 = '';
+	$l2 = '';
+
+	#
+	#	If it looks like hex, only try it as hex,
+	#	otherwise try it as both decimal and hex.
+	#
+	$sess_id = $ARGV[4];
+	if ($sess_id !~ /^0/ && $sess_id !~ /[a-f]/i) {
+		$l1 = snmpget($ARGV[1], $ARGV[5], "$asm.12.3.1.4.$sess_id");
+	}
+	$sess_id = hex $ARGV[4];
+	$l2 = snmpget($ARGV[1], $ARGV[5], "$asm.12.3.1.4.$sess_id");
+
+	print LOG "  user at port S$ARGV[2]: $l1 (dec)\n" if ($debug && $l1);
+	print LOG "  user at port S$ARGV[2]: $l2 (hex)\n" if ($debug && $l2);
+
+	(($l1 && $l1 eq $ARGV[3]) || ($l2 && $l2 eq $ARGV[3])) ? 1 : 0;
+}
+  
+
 #
 #	See if the user is logged in using the portslave finger.
 #
@@ -509,7 +545,7 @@ sub tc_tccheck {
 #
 #	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
-sub pathras_telnet {
+sub cyclades_telnet {
    #
    #	Localize all variables first.
    #
@@ -628,7 +664,7 @@ sub patton_snmp {
    #
    # Check if the session still active
    # 
-   if (snmpget($ARGV[1], "monitor", "$oid") == 0) {
+   if (snmpget($ARGV[1], $ARGV[5], "$oid") == 0) {
       print LOG "  Session $ARGV[4] still active on NAS " . 
       	"$ARGV[1], port $ARGV[2], for user $ARGV[3].\n" if ($debug);
       return 1;
@@ -639,26 +675,54 @@ sub patton_snmp {
 #
 #      Check a Digitro BXS via rusers
 #
-#      Version: 1.0
+#      Version: 1.1
 #
 #      Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
 sub digitro_rusers {
+   my ($ret);
+   local $_;
+
    if (-e $rusers && -x $rusers) {
-      open(FD, "$rusers $ARGV[1]|");
+      #
+      # Get a list of users logged in via rusers
+      #
+      $_ =`$rusers $ARGV[1]`;
+      $ret = ((/$ARGV[3]/) ? 1 : 0);
    } else {
       print LOG "   Error: can't execute $rusers\n" if $debug;
-      return 2;
+      $ret = 2;
    }
+   $ret;
+}
 
-   while(<FD>) {
-      if (/$ARGV[3]/) {
-	close FD;
-	return 1;
-      }
+#
+#	Check Cyclades PR3000 and PR4000 via SNMP
+#
+#	Version: 1.0
+#
+#	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
+#
+sub cyclades_snmp {
+   my ($oid, $ret);
+   local $_;
+   
+   $oid = ".1.3.6.1.4.1.2925.3.3.6.1.1.2";
+
+   if (-e $snmpwalk && -x $snmpwalk) { 	
+      #
+      # Get a list of logged in users via snmpwalk
+      #
+      # I'm looking for a way to do this using SNMP_Session
+      # module. Any help would be great.
+      #
+      $_ = `$snmpwalk $ARGV[1] $ARGV[5] $oid`;
+      $ret = ((/$ARGV[3]/) ? 1 : 0);
+   } else {
+      print LOG "   Error: can't execute $snmpwalk\n" if $debug;
+      $ret = 2;
    }
-   close FD;
-   0;
+   $ret;
 }
 
 #
@@ -674,7 +738,11 @@ sub usrhiper_snmp {
 	my ($login);
 
 	$login = snmpget($ARGV[1], $ARGV[5], "$usrm.4.10.1.1.18.$oidext");
-	($login) = /^.*\"([^"]+)".*$/;
+	#($login) = /^.*\"([^"]+)".*$/;
+	if ($login =~ /\"/) {
+		$login =~ /^.*\"([^"]+)\"/;
+		$login = $1;
+	}
 
 	print LOG "  user at port S$ARGV[2]: $login\n" if ($debug);
 
@@ -779,10 +847,7 @@ sub versanet_snmp {
 	$slot = (($ARGV[2]-$port)/8)+1;
 	print LOG "slot = $slot" if ($debug);
 	$loginname = snmpget($ARGV[1], $ARGV[5], "$vsm.27.1.1.3.$slot.$port");
-#
-#	Note: the $ARGV[5] above could be replaced by the public
-#	      community string defined in Versanet VN2001/VN2002.
-#
+
 	  print LOG "  user at slot $slot port $port: $loginname\n" if ($debug);	  ($loginname eq $ARGV[3]) ? 1 : 0;	
 }
 
@@ -793,11 +858,11 @@ if ($debug) {
 	print LOG "$now checkrad @ARGV\n";
 }
 
-if ($#ARGV != 5) {
+if ($#ARGV < 4) {
 	print LOG "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id community\n" if ($debug);
+			"nas_port login session_id [community]\n" if ($debug);
 	print STDERR "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id community\n";
+			"nas_port login session_id [community]\n";
 	close LOG if ($debug);
 	exit(2);
 }
@@ -812,12 +877,18 @@ if ($ARGV[0] eq 'livingston') {
 	$ret = &computone_finger;
 } elsif ($ARGV[0] eq 'max40xx') {
 	$ret = &max40xx_finger;
+} elsif ($ARGV[0] eq 'ascend') {
+	$ret = &ascend_snmp;
 } elsif ($ARGV[0] eq 'portslave') {
 	$ret = &portslave_finger;
 } elsif ($ARGV[0] eq 'tc') {
 	$ret = &tc_tccheck;
 } elsif ($ARGV[0] eq 'pathras') {
-	$ret = &pathras_telnet;
+	$ret = &cyclades_telnet;
+} elsif ($ARGV[0] eq 'pr3000') {
+	$ret = &cyclades_snmp;
+} elsif ($ARGV[0] eq 'pr4000') {
+	$ret = &cyclades_snmp;
 } elsif ($ARGV[0] eq 'patton') {
 	$ret = &patton_snmp;
 } elsif ($ARGV[0] eq 'digitro') {
