@@ -37,12 +37,6 @@ char dict_sccsid[] =
 
 #include	"radiusd.h"
 
-#ifndef USEMYSQL
-static DICT_ATTR	*dictionary_attributes;
-static DICT_VALUE	*dictionary_values;
-static DICT_VENDOR	*dictionary_vendors;
-#endif
-
 static int		vendorno = 1;
 
 #ifdef NOCASE
