@@ -62,6 +62,7 @@ int sql_init(int reload) {
 	sql->config->sensitiveusername = 1;
 	sql->config->deletestalesessions = 0;
 
+	strcpy(sql->config->sql_hints_table,"hints");
 	strcpy(sql->config->sql_nas_table,"nas");
 	strcpy(sql->config->sql_realm_table, "realms");
 	strcpy(sql->config->sql_dict_table,"dictionary");
@@ -169,6 +170,14 @@ int sql_init(int reload) {
                        } else {
                          strncpy(sql->config->sql_acct_table,namestr, MAX_TABLE_LEN);
                        }
+               }
+                if(strncasecmp(buffer, "hints_table", 11) == 0) {
+                        /* Read the HINTS_TABLE line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         strncpy(sql->config->sql_hints_table,namestr, MAX_TABLE_LEN);
+                       }
                }
                 if(strncasecmp(buffer, "nas_table", 9) == 0) {
                         /* Read the NAS_TABLE line */
@@ -389,7 +398,7 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 #endif /* NT_DOMAIN_HACK */
 
 	// keep garbled usernames from messing up our database queries
-	sql_escape_string(fixeduser, sqlrecord->UserName, strlen(sqlrecord->UserName));
+	sql_escape_string(socket, fixeduser, sqlrecord->UserName, strlen(sqlrecord->UserName));
 	strNcpy(sqlrecord->UserName, fixeduser, SQLBIGREC);
 
      if (sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
@@ -430,13 +439,14 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
           if(sqlrecord->AcctStatusTypeId == PW_STATUS_START) {
              
              /* Set start time on record with only a stop record */
- 	     snprintf(querystr, 2048, "UPDATE %s SET AcctStartTime = '%s', AcctStartDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s'", 
+ 	     snprintf(querystr, 2048, "UPDATE %s SET AcctStartTime = '%s', AcctStartDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
 	     sql->config->sql_acct_table,
              sqlrecord->AcctTimeStamp,
 	     sqlrecord->AcctDelayTime,
              sqlrecord->AcctSessionId,
              sqlrecord->UserName,
-             sqlrecord->NASIPAddress
+             sqlrecord->NASIPAddress,
+             sqlrecord->NASPortId
              );
        	     if (sql_query(socket, querystr) < 0)
 	        log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", sql_error(socket));
@@ -473,9 +483,9 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
            /* Got stop record */
            } else {
 
-		char	authstr[AUTH_STRING_LEN+30];
+		char	authstr[(AUTH_STRING_LEN * 2) + 50];
 		if (sql->config->sensitiveusername) 
-			sprintf(authstr, "STRCMP(Username, '%s') = 0", sqlrecord->UserName);
+			sprintf(authstr, "UserName = '%s' AND STRCMP(UserName, '%s') = 0", sqlrecord->UserName, sqlrecord->UserName);
 		else
 			sprintf(authstr, "UserName = '%s'", sqlrecord->UserName);
              sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND %s", sql->config->sql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, authstr);
@@ -486,7 +496,7 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
              if (num > 0) {
 
                 /* Set stop time on matching record with start time */
- 	        snprintf(querystr, 2048, "UPDATE %s SET AcctStopTime = '%s', AcctSessionTime = '%lu', AcctInputOctets = '%lu', AcctOutputOctets = '%lu', AcctTerminateCause = '%s', AcctStopDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s'", 
+ 	        snprintf(querystr, 2048, "UPDATE %s SET AcctStopTime = '%s', AcctSessionTime = '%lu', AcctInputOctets = '%lu', AcctOutputOctets = '%lu', AcctTerminateCause = '%s', AcctStopDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
 	        sql->config->sql_acct_table,
                 sqlrecord->AcctTimeStamp,
                 sqlrecord->AcctSessionTime,
@@ -496,7 +506,8 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 		sqlrecord->AcctDelayTime,
                 sqlrecord->AcctSessionId,
                 sqlrecord->UserName,
-                sqlrecord->NASIPAddress
+                sqlrecord->NASIPAddress,
+                sqlrecord->NASPortId
                 );
 
 
@@ -566,7 +577,7 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
  *************************************************************************/
 int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 
-	int		x, ufound, gfound;
+	int		x, ufound, gfound, hintsfound;
 	char		*s;
 	DICT_ATTR	*attr = NULL;
 	DICT_VALUE	*dval;
@@ -585,14 +596,21 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 	check = *first_pair;
 	ufound = 0;
 	gfound = 0;
+	hintsfound = 0;
 	while (check) {
 		if (attr->value == check->attribute) {
+			if (check->source == PW_VP_HINTSDATA || mode == PW_VP_HINTSDATA)
+				hintsfound = 1;
 			if (check->source == PW_VP_USERDATA || mode == PW_VP_USERDATA)
 				ufound = 1;
 			if (check->source == PW_VP_GROUPDATA || mode == PW_VP_GROUPDATA)
 				gfound = 1;
 		}
 		
+		/* HINTS items ALWAYS replace user or group items */
+		if (hintsfound && (ufound || gfound)) 
+			return 0;
+
 		if (ufound && gfound)
 			return 0;
 
@@ -714,9 +732,9 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
  *************************************************************************/
 int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int mode) {
 
-	char		querystr[256];
-	char		authstr[256];
-	char		username[AUTH_STRING_LEN*2+1];
+	char		*querystr;
+	char		*authstr;
+	char		*username;
 	SQL_ROW		row;
 	int		rows;
 	int		length;
@@ -726,23 +744,59 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 	else
 		length = strlen(user);
 
-	/* FIXME CHECK user for weird charactors!! */
-	sql_escape_string(username, user, length);
+	if ((username = malloc(length* 2 + 1)) == NULL) {
+		log(L_CONS|L_ERR, "out of memory");
+		return -1;
+	}
+
+	sql_escape_string(socket, username, user, length);
 
 	if (mode == PW_VP_USERDATA) {
+		if ((authstr = malloc((strlen(username) * 2) + 50)) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		if (sql->config->sensitiveusername) 
-			sprintf(authstr, "STRCMP(Username, '%s') = 0", username);
+			sprintf(authstr, "UserName = '%s' AND STRCMP(Username, '%s') = 0", username, username);
 		else
 			sprintf(authstr, "UserName = '%s'", username);
+		if ((querystr = malloc(strlen(authstr) + strlen(table) + 70)) == NULL) {
+			free(authstr);
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		sprintf(querystr, "SELECT id, UserName, Attribute, Value FROM %s WHERE %s ORDER BY id", table, authstr);
+
 	} else if (mode == PW_VP_GROUPDATA) {
+		if ((authstr = malloc((strlen(username) * 2) + strlen(sql->config->sql_usergroup_table) + 55)) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		if (sql->config->sensitiveusername) 
-			sprintf(authstr, "STRCMP(%s.Username, '%s') = 0", sql->config->sql_usergroup_table, username);
+			sprintf(authstr, "%s.UserName = '%s' AND STRCMP(%s.Username, '%s') = 0", sql->config->sql_usergroup_table, username, sql->config->sql_usergroup_table, username);
 		else
 			sprintf(authstr, "%s.UserName = '%s'",sql->config->sql_usergroup_table, username);
+		if ((querystr = malloc(strlen(authstr) + (strlen(table) * 7) + (strlen(sql->config->sql_usergroup_table) * 2) + 125)) == NULL) {
+			free(authstr);
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		sprintf(querystr, "SELECT %s.id, %s.GroupName, %s.Attribute, %s.Value FROM %s, %s WHERE %s AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, table, table, table, sql->config->sql_usergroup_table, authstr, sql->config->sql_usergroup_table, table, table);
+
 	} else if (mode == PW_VP_REALMDATA) {
+		if ((querystr = malloc(strlen(username) + (strlen(table) * 7) + (strlen(sql->config->sql_realmgroup_table) * 3) + 125)) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		sprintf(querystr, "SELECT %s.id, %s.GroupName, %s.Attribute, %s.Value FROM %s, %s WHERE %s.RealmName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, table, table, table, sql->config->sql_realmgroup_table, sql->config->sql_realmgroup_table, username, sql->config->sql_realmgroup_table, table, table);
+	} else if (mode == PW_VP_HINTSDATA) {
+		if ((querystr = malloc(strlen(username) + strlen(table) + 125)) == NULL) {
+			free(authstr);
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
+		sprintf(querystr, "SELECT id, GroupName, Attribute, Value FROM %s WHERE GroupName = '%s' ORDER BY id", table, username);
+		
 	}
 
 	sql_select_query(socket, querystr);
@@ -752,13 +806,17 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 		if (sql_userparse(vp, row, mode) != 0) {
 	 		log(L_ERR|L_CONS, "sql_userparse: Error adding row to pair list");
 			sql_finish_select_query(socket);
+			free(querystr);
+			free(authstr);
 			return -1;
 		}
 	}
 	sql_finish_select_query(socket);
+	free(querystr);
+	free(authstr);
+	free(username);
 
 	return rows;
-
 }
 
 
@@ -876,7 +934,7 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 	int		mpp = 1;
 
 	if (sql->config->sensitiveusername)
-		sprintf(authstr, "STRCMP(UserName, '%s') = 0", name);
+		sprintf(authstr, "UserName = '%s' AND STRCMP(UserName, '%s') = 0", name, name);
 	else
 		sprintf(authstr, "UserName = '%s'", name);
 	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE AcctStopTime = 0 AND %s", sql->config->sql_acct_table, authstr);
@@ -935,85 +993,127 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 
 }
 
+/*
+ *      Read the hints table
+ */
+PAIR_LIST  *sql_read_hints(SQLSOCK *socket) {
+	char		querystr[256];
+	PAIR_LIST	*c, *last = NULL, *pl = NULL;
+	VALUE_PAIR	*check_tmp = NULL;
+	VALUE_PAIR	*reply_tmp = NULL;
+	SQL_ROW		row;
+	SQLSOCK		*hintsock;
+
+	hintsock = sql_get_socket();
+
+
+	sprintf(querystr, "SELECT id, HintName, GroupName FROM %s", sql->config->sql_hints_table);
+	sql_select_query(socket, querystr);
+	while ((row = sql_fetch_row(socket))) {
+		if ((c = malloc(sizeof(PAIR_LIST))) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			exit(1);
+		}
+
+		c->next = NULL;
+
+		/* Why the HELL was strdup(row[1]) not working?! */
+		if ((c->name = malloc(strlen(row[1]) + 1)) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			exit(1);
+		}
+		strcpy(c->name, row[1]);
+
+		c->lineno = atoi(row[0]);
+
+		sql_getvpdata(hintsock, sql->config->sql_groupcheck_table, &check_tmp, row[2], PW_VP_HINTSDATA);
+		sql_getvpdata(hintsock, sql->config->sql_groupreply_table, &reply_tmp, row[2], PW_VP_HINTSDATA);
+
+		c->check = check_tmp;
+		c->reply = reply_tmp;
+		check_tmp = NULL;
+		reply_tmp = NULL;
+
+#if 0
+		debug_pair_list(c);
+#endif
+
+		if (last)
+			last->next = c;
+		else
+			pl = c;
+		last = c;
+       	}
+	sql_finish_select_query(socket);
+
+	sql_release_socket(hintsock);
+
+        return pl;
+} 
+
 
 /*
  *      Read the nas table
  */
 int sql_read_naslist(SQLSOCK *socket) {
 	char		querystr[256];
-        NAS     	*c;
-	SQL_ROW		row;
-
-        nas_free(naslist);
-        naslist = NULL;
-
-	sprintf(querystr, "SELECT * FROM %s", sql->config->sql_nas_table);
-	sql_select_query(socket, querystr);
-	while ((row = sql_fetch_row(socket))) {
-		if ((c = malloc(sizeof(NAS))) == NULL) {
-			log(L_CONS|L_ERR, "out of memory");
-			return -1;
-		}
-
-		c->longname[0] = 0;
-		c->shortname[0] = 0;
-		c->community[0] = 0;
-		c->usesnmp = 0;
-		if (row[1] != NULL)
-               		strNcpy(c->longname, row[1], sizeof(c->longname));
-		if (row[2] != NULL)
-               		strNcpy(c->shortname, row[2], sizeof(c->shortname));
-               	c->ipaddr = ipstr2long(row[3]);
-		if (row[4] != NULL)
-                	strNcpy(c->nastype, row[4], sizeof(c->nastype));
-		if (row[7] != NULL) {
-			strncpy(c->community, row[7], MAX_COMMUNITY_LEN);
-		}
-		if (row[8] != NULL)	
-			c->usesnmp = strcmp(row[8], "ON") == 0 ||
-				     strcmp(row[8], "on") == 0;
-
-       	        c->next = naslist;
-               	naslist = c;
-       	}
-	sql_finish_select_query(socket);
-
-        return 0;
-} 
-
-
-/*
- *      Read the nas table into CLIENT struct
- */
-int sql_read_clients(SQLSOCK *socket) {
-	char		querystr[256];
-        CLIENT     	*c;
+        NAS     	*n;
+	CLIENT		*c;
 	SQL_ROW		row;
 
         clients_free(clients);
         clients = NULL;
 
-	sprintf(querystr, "SELECT * FROM %s", sql->config->sql_nas_table);
+        nas_free(naslist);
+        naslist = NULL;
+
+	sprintf(querystr, "SELECT id, nasname, shortname, ipaddr, type, ports, secret, community, snmp FROM %s", sql->config->sql_nas_table);
 	sql_select_query(socket, querystr);
 	while ((row = sql_fetch_row(socket))) {
+		if ((n = malloc(sizeof(NAS))) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
 		if ((c = malloc(sizeof(CLIENT))) == NULL) {
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
 
 		c->secret[0] = 0;
-		c->shortname[0] = 0;
 		c->longname[0] = 0;
+		c->shortname[0] = 0;
+
+		n->longname[0] = 0;
+		n->shortname[0] = 0;
+		n->community[0] = 0;
+		n->usesnmp = 0;
+
+               	n->ipaddr = ipstr2long(row[3]);
                	c->ipaddr = ipstr2long(row[3]);
-		if (row[1] != NULL)
+
+		if (row[1] != NULL) {
+               		strNcpy(n->longname, row[1], sizeof(n->longname));
                		strNcpy(c->longname, row[1], sizeof(c->longname));
-		if (row[2] != NULL)
+		}
+		if (row[2] != NULL) {
+               		strNcpy(n->shortname, row[2], sizeof(n->shortname));
                		strNcpy(c->shortname, row[2], sizeof(c->shortname));
+		}
+		if (row[4] != NULL)
+                	strNcpy(n->nastype, row[4], sizeof(n->nastype));
 		if (row[6] != NULL)
                 	strNcpy(c->secret, row[6], sizeof(c->secret));
+		if (row[7] != NULL)
+			strncpy(n->community, row[7], MAX_COMMUNITY_LEN);
+		if (row[8] != NULL)	
+			n->usesnmp = strcmp(row[8], "ON") == 0 ||
+				     strcmp(row[8], "on") == 0;
 
        	        c->next = clients;
                	clients = c;
+
+       	        n->next = naslist;
+               	naslist = n;
        	}
 	sql_finish_select_query(socket);
 
@@ -1305,7 +1405,6 @@ int sql_read_realms(SQLSOCK *socket) {
 	c->next = realms;
 	realms = c;
     }
-       
     sql_finish_select_query(socket);
     return 1;
 }
