@@ -373,7 +373,7 @@ int main(int argc, char **argv)
 		if(pid > 0) {
 			exit(0);
 		}
-#if defined(__linux__) || defined(__svr4__)
+#if defined(__linux__) || defined(__svr4__) || defined(__hpux__)
 		setsid();
 #endif
 	}
