@@ -85,6 +85,8 @@ typedef struct auth_req {
 	u_char			*data;		/* Raw received data */
 	int			data_len;
 	VALUE_PAIR		*proxy_pairs;
+	VALUE_PAIR		*check_pairs;
+	VALUE_PAIR		*reply_pairs;
 	/* Proxy support fields */
 	u_char			realm[32];
 	int			validated;	/* Already md5 checked */
@@ -281,21 +283,19 @@ int		calc_digest (u_char *, AUTH_REQ *);
 int		calc_acctdigest(u_char *digest, AUTH_REQ *authreq);
 
 /* files.c */
-int		user_find(char *name, VALUE_PAIR *,
-				VALUE_PAIR **, VALUE_PAIR **, SQLSOCK *socket);
+VALUE_PAIR	*paircopy(VALUE_PAIR *from);
+int		user_find(char *name, AUTH_REQ *authreq, SQLSOCK *socket);
 int		userparse(char *buffer, VALUE_PAIR **first_pair);
 void		presuf_setup(VALUE_PAIR *request_pairs);
-int		hints_setup(VALUE_PAIR *request_pairs);
+int		hints_setup(AUTH_REQ *authreq);
 int		huntgroup_access(VALUE_PAIR *request_pairs);
 CLIENT		*client_find(UINT4 ipno);
 char		*client_name(UINT4 ipno);
-int		read_clients_file(char *);
 REALM		*realm_find(char *);
 NAS		*nas_find(UINT4 ipno);
 char		*nas_name(UINT4 ipno);
 char		*nas_name2(AUTH_REQ *r);
 char		*auth_name(AUTH_REQ *authreq, int do_cid);
-int		read_naslist_file(char *);
 int		read_config_files(SQLSOCK *socket);
 int		paircmp(VALUE_PAIR *request, VALUE_PAIR *check);
 int		presufcmp(VALUE_PAIR *check, char *name, char *rest, int rl);
@@ -346,6 +346,6 @@ int		sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode);
 int		sql_read_realms(SQLSOCK *socket);
 int		sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int mode);
 int		sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsimul);
+PAIR_LIST	*sql_read_hints(SQLSOCK *socket);
 int		sql_read_naslist(SQLSOCK *socket);
-int		sql_read_clients(SQLSOCK *socket);
 int		sql_dict_init(SQLSOCK *socket);
