@@ -244,8 +244,11 @@ int sql_affected_rows(SQLSOCK *socket) {
  *      Purpose: Esacpe "'" and any other wierd charactors
  *
  *************************************************************************/
-int sql_escape_string(char *to, char *from, int length) {
+int sql_escape_string(SQLSOCK *socket, char *to, char *from, int length) {
 
-	mysql_escape_string(to, from, length);
-	return 1;
+#if MYSQL_VERSION_ID >= 32315
+	return mysql_real_escape_string(socket->sock, to, from, length);
+#else
+	return mysql_escape_string(to, from, length);
+#endif
 }
